/*
 * API for the Wagon race
 *
 *
 * *** Endpoint Documentation ***
 * Endpoint: /new_race
 * Description: chooses num_drivers random drivers. The drivers and which wagon
 *              they drive are stored in files under the wagons/ directory.
 * Request Type: GET
 * Query Parameters: num_drivers: How many drivers. Must be >= 0 and <= number of wagons available
 * Response Type: JSON
 * Example Request: /new_race?num_wagons=2
 * Example Response:
 *   {
 *    "wagons": [
 *      { "driver": "Jack", "color": "dark_green", "capacity": 1024, "speed": 1 },
 *      { "driver": "Tal", "color": "purple", "capacity": 2048, "speed": 1.1 }
 *    ],
 *    "distance": 10
 *   }
 * Error Handling: Responds in JSON with a 400 if the required parameters are not passed correctly
 *                 Responds in JSON with a 500 error if it can't create a new race (error reading files).
 * *************************************
 * Endpoint: /winner
 * Description: Record the winner of the race.
 * Request Type: POST
 * Required Parameters: driver AND/OR wagon_color
 * Response Type: Text
 * Example Request: /winner {driver: "Fitz", wagon_color: "neon_green"}
 * Example Response: The winner has been recorded!
 * Error Handling: Responds in text with 400 if the required parameters are not passed.
 */

"use strict";

const express = require('express');
const fs = require('fs').promises;
const util = require('util');
const multer = require('multer');
const glob = require('glob');

const globPromise = util.promisify(glob);

const app = express();

app.use(express.urlencoded({ extended: true }));
app.use(express.json());
app.use(multer().none());

/**
 *  Endpoint to start a new race
 *  Will send back (as JSON) the shuffled number of wagons and the distance
 */
app.get('/new_race', async (req, res) => {
  if (req.query['num_wagons'] === undefined ) {
    let msg = "Invalid parameter";
     console.error(msg);
     res.status(400).json({ "error": msg });
  }
  else {
    let distance = Math.random() * 3000 + 1;

    let respJson = {
      "wagons": [],
      "distance": distance
    };

    let numWagons = parseInt(req.query['num_wagons']);
    let wagons = null;
    try {
      wagons = await getWagons(numWagons);
    } catch (err) {
      let msg = "Cannot create new race";
      console.error(msg);
      res.status(500).json({ "error": msg });
    }
    if  (wagons === null || numWagons <= 0 || wagons.length < numWagons) {
      let msg = "Invalid parameter";
      console.error(msg);
      res.status(400).json({ "error": msg });
    }
    else {
      respJson["wagons"] = wagons;
      res.json(respJson);
    }
  }
});

/**
 *  Endpoint to record the driver and color of the wagon as the winner.
 *  Will send back a text response if this it worked
 */
app.post('/winner', async (req, res) => {
  let winner = req.body['driver'];
  let color = req.body['color'];
  let msg = winner + " in a " + color + " wagon.";
  if (winner === undefined || color === undefined ) {
    res.status(400).send("Could not record winner: " + msg);
  }
  else if (recordWinner(winner, color)) {
    res.type('text').send("Recorded winner: " + msg);
  }
  else {
    res.status(500).send("Could not record winner: " + msg);
  }
});

/**
 *  Start a new race by getting the shuffled number of wagons and the distance
 *  @param {number} numWagons The number of wagons to return
 *  @return {Array} An array of numWagon JSON objects, each entry in the array respresents
 *                  a wagon in the race.
 */
async function getWagons(numWagons) {
    let wagons = [];
    let matches = await globPromise("wagons/*.txt");

    for (let i = 0; i < matches.length; i++) {
      let contents = await fs.readFile(matches[i], "utf-8");
      let data = contents.split(' ');
      let wagon = {
        "driver": data[0],
        "color": data[1],
        "capacity": data[2],
        "speed": data[3]
      };
      wagons.push(wagon);
    }

    wagons = shuffle(wagons);
    wagons = wagons.slice(0, numWagons);
    return wagons;
}




/**** GIVEN FUNCTIONS ****/


/**
 * Takes an array and returns an array with the same elements in a random order.
 * @param {array} array the array to shuffle
 */
function shuffle(array) {
  /*
  Shamelessly copied from
  https://stackoverflow.com/questions/2450954/how-to-randomize-shuffle-a-javascript-array

  Thanks, StackOverflow!! =D
  */
  var currentIndex = array.length, temporaryValue, randomIndex;

  // While there remain elements to shuffle...
  while (0 !== currentIndex) {

    // Pick a remaining element...
    randomIndex = Math.floor(Math.random() * currentIndex);
    currentIndex -= 1;

    // And swap it with the current element.
    temporaryValue = array[currentIndex];
    array[currentIndex] = array[randomIndex];
    array[randomIndex] = temporaryValue;
  }

  return array;
}


/**
 * "Record the winner" - should be done into a DB or a file. For now we'll randomly
 * roll the dice and throw an error 1/4 of the time. Assumes both parameters are non empty strings
 * @param {string} driver The name of the driver to be recorded as a winner
 * @param {string} wagonColor The color of the wagon
 * @return {boolean} whether or not the winner was sucessfully recorded
 */
function recordWinner(driver, wagonColor) {
  let dieRoll = Math.floor(Math.random() * 4);
  return dieRoll !== 1;
}


app.use(express.static("public"));
const PORT = process.env.PORT || 8886;
app.listen(PORT);
