/*
 *  index.js for the Wagon Race web page.
 *
 *  Ready Set GO! Get some wagons from the Wagon Race API and run a race
 *  Record the winner when it's done
 */

"use strict";

(function () {
  window.addEventListener("load", init);


  /** Module global to keep track of the interval (timer) handle so we can stop it later */
  let activeRace = null;

  /**
   * Initialize the page, really just the button on the page...
   */
  function init() {
    id("start-race").addEventListener('click', newRace);
  }

  /**
   * Initialize the race by getting the wagsons from the API
   */
  function newRace() {
    let numWagons = 'num_wagons=' + id('num-wagons').value;
    id("tracker").innerHTML = "";
    id('num-wagons').value = "";

    fetch('/new_race?' + numWagons)
      .then(checkStatus)
      .then(resp => resp.json())
      .then(startRace)
      .catch(console.error);
  }

  /**
   * Respond to an API call that is returning all of the cars as a JSON object then
   * place the cars on the screen ...
   * @param {Object} A JSON object from the API which has a list of wagons and the distance.
   */
  function startRace(data) {
    let wagons = data["wagons"];

    for (let i = 0; i < wagons.length; i++) {
      placeCar(i, wagons[i]["color"], wagons[i]["driver"], wagons[i]["speed"]);
    }

    activeRace = setInterval(moveCars, 500);
  }


  /**
   * Add a car to the page on the left hand side of the "race track"
   * @param {number} index The number of the wagon in the list of wagons (top to bottom).
   *                       Must be an integer.
   * @param {string} color The color of the wagon
   * @param {string} driver The driver of the wagon
   * @param {number} speed The speed at which the wagon can travel.
   */
  function placeCar(index, color, driver, speed) {
    let container = id("tracker");
    let car = gen("img");
    car.src = "img/" + color + "_wagon.png";
    car.alt = color + " Wagon";
    car.classList.add("wagon");
    car.id = color + " - " + driver + " - " + speed;

    container.appendChild(car);

    car.style.top = getCarTopPx(index);

    // Could add this
    // car.addEventListener("click", boostCar);
  }


  /**
   * Moves the cars along in the race for every tick of the interval. As soon as
   * any of the cars hits the "finish" line (the right side of the window), stop the
   * timer and record the winner. Some cars may move after the winner has been declared.
   */
  function moveCars() {
    let cars = qsa(".wagon");
    for (let i = 0; i < cars.length; i++) {
      let moved = moveACar(cars[i]);
      if (!moved) {
        clearInterval(activeRace);
        recordWinner(cars[i]);
      }
    }
  }


  /**
   * Sends the winning car to the API for recording.
   * @param {Object} car The DOM element of the car that won the race
   */
  function recordWinner(car) {
    let info = car.id.split('-');
    let params = new FormData();
    params.append("color", info[0]);
    params.append("driver", info[1]);
    fetch('/winner', { method: "POST", body: params })
      .then(checkStatus)
      .catch(console.error);
  }




  /**** GIVEN FUNCTIONS ****/


  /**
   * Moves the given car
   * @param {Element} car
   * @returns {boolean} true if the car finished, false otherwise.
   */
  function moveACar(car) {
    let containerWidth = parseInt(getComputedStyle(id("tracker")).width);
    let carWidth = parseInt(getComputedStyle(qs(".wagon")).width);

    let speed = car.id.split("-")[2];
    let percentToMove = speed / 100;

    let oldLeftPx = parseInt(getComputedStyle(car).left);
    let newLeftPx = oldLeftPx + (containerWidth * percentToMove);

    let finished = false;
    if ((newLeftPx + carWidth) < containerWidth) {
      car.style.left = newLeftPx + "px";
      finished = true;
    } else {
      car.style.left = (containerWidth - carWidth) + "px";
    }

    return finished;
  }



  /**
   * Returns how far from the top of the tracker this car should be.
   * @param {number} index A unique index for this car
   * @returns {string} A relative pixel position. Example: "10px"
   */
  function getCarTopPx(index) {
    let container = id("tracker");
    let anyCar = qs(".wagon");
    let containerHeight = parseInt(getComputedStyle(container).height);
    let carHeight = parseInt(getComputedStyle(anyCar).height);
    return (index * containerHeight / carHeight) + "px";
  }


  /**
   * The car was clicked! Assumed the "this" is the car that was clicked so give it a boost.
   */
  function boostCar() {
    moveCar(this);
  }


  /**
   * Updates the stats table with the statistics for the current race.
   */
  function showWinner() {

  }

  function updateFormSuccess(formId) {
    let form = id(formId);

    updateForm(form, "success");

    let inputs = form.querySelectorAll("input");
    for (let i = 0; i < inputs.length; i++) {
      inputs[i].value = '';
    }
  }

  function updateFormFailure(error, formId) {
    console.error(error);
    updateForm(id(formId), "fail");
  }

  function updateForm(form, type) {
    let q;
    if (type === "fail") {
      q = ".red_x";
    } else if (type === "success") {
      q = ".green_check";
    } else {
      return;
    }

    let check = form.querySelector(q);
    check.classList.toggle("hidden");

    setTimeout(() => check.classList.toggle("hidden"), 5000);
  }

  function checkStatus(response) {
    if (!response.ok) {
      throw Error("Error in request: " + response.statusText);
    }
    return response; // a Response object
  }

  /**
   * Returns the element that has the ID attribute with the specified value.
   * @param {string} idName - element ID
   * @returns {object} DOM object associated with id.
   */
  function id(idName) {
    return document.getElementById(idName);
  }

  /**
   * Returns the first element that matches the given CSS selector.
   * @param {string} selector - CSS query selector.
   * @returns {object} The first DOM object matching the query.
   */
  function qs(selector) {
    return document.querySelector(selector);
  }

  /**
   * Returns the array of elements that match the given CSS selector.
   * @param {string} selector - CSS query selector
   * @returns {object[]} array of DOM objects matching the query.
   */
  function qsa(selector) {
    return document.querySelectorAll(selector);
  }

  /**
   * Returns a new element with the given tag name.
   * @param {string} tagName - HTML tag name for new DOM element.
   * @returns {object} New DOM object for given HTML tag.
   */
  function gen(tagName) {
    return document.createElement(tagName);
  }
})();
